/**
 * 
 * Copyright 2004 Protique Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.activemq.openwire.tool;

import org.codehaus.gram.GramSupport;
import org.codehaus.jam.JClass;
import org.codehaus.jam.JMethod;
import org.codehaus.jam.JProperty;

import java.util.HashMap;
import java.util.Map;

/**
 * @version $Revision: 1.1 $
 */
public abstract class OpenWireScript extends GramSupport {

    private Map enums = new HashMap();
    private int enumCounter;
    private Map testCaseValues = new HashMap();
    private int counter;


    /**
     * Returns true if the given type is a valid message type
     */
    public boolean isMessageType(JClass it) {
        // lets filter out silly non-qualified classes
        if (it.getSimpleName().equals(it.getQualifiedName())) {
            return false;
        }
        String name = it.getSimpleName();

        return it.isInterface() == false && it.isAbstract() == false && !name.startsWith("Abstract") &&
                name.indexOf("Queue") < 0 && name.indexOf("Topic") < 0
                && !name.startsWith("ActiveMQDestination")
                && !name.startsWith("ActiveMQXid")
                && !name.startsWith("MessageAcknowledge")
                && !name.startsWith("Packet")
                && !name.startsWith("ReceiptHolder")
                && !name.startsWith("TransactionType");
    }

    public boolean isBodyType(JClass type, JProperty property) {
        String name = property.getType().getQualifiedName();
        String typeName = type.getQualifiedName();
        if (name.equals("java.lang.Object") || name.equals("java.io.Serializable")) {
            return true;
        }
        if (name.equals("java.lang.String") && typeName.equals("org.activemq.message.ActiveMQTextMessage")) {
            return true;
        }
        return false;
    }

    /**
     * Creates a new test case value for the given named property.
     *
     * @return
     */
    public Object createTestValue(JClass packetType, JProperty property) {
        JClass type = property.getType();
        String name = type.getQualifiedName();
        ++counter;
        if (type.getQualifiedName().equals("java.lang.Object")) {
            return "\"DummyString" + counter + "\"";
        }
        else if (property.getGetter().getSimpleName().equals("getConsumerNosAsString")) {
            return "\"1," + counter + "\"";
        }
        else if (name.equals("java.lang.String")) {
            return "\"TestString" + counter + "\"";
        }
        else if (isDestinationType(name)) {
            return "new ActiveMQQueue(\"TEST.FOO.BAR." + counter + "\")";
        }
        else if (isPropertiesType(name)) {
            return null;
        }
        else if (name.equals("boolean")) {
            return counter % 2 == 1 ? Boolean.TRUE : Boolean.FALSE;
        }
        else if (name.equals("byte")) {
            return "(byte) " + counter;
        }
        else if (name.equals("char")) {
            return "(char) " + counter;
        }
        else if (name.equals("short")) {
            return "(short) " + counter;
        }
        else if (name.equals("int")) {
            return new Integer(counter);
        }
        else if (name.equals("long")) {
            return "(long) " + counter;
        }
        else if (name.equals("float")) {
            return "(float) " + counter;
        }
        else if (name.equals("double")) {
            return "(double) " + counter;
        }
        return "null";
    }

    public boolean isValidProperty(JProperty it) {
        JMethod getter = it.getGetter();
        return getter != null && it.getSetter() != null && getter.isPublic() && getter.isStatic() == false && getter.getAnnotation("Transient") == null;
    }

    public boolean isPropertiesType(String type) {
        return "java.util.Properties".equals(type) || "java.util.Map".equals(type);
    }

    public boolean isDestinationType(String type) {
        return "org.activemq.message.ActiveMQDestination".equals(type) || "javax.jms.Destination".equals(type);
    }

    /**
     * Returns the enum counter for the given type
     */
    public int getEnum(JClass type) {
        Integer value = (Integer) enums.get(type);
        if (value == null) {
            value = new Integer(++enumCounter);
            enums.put(type, value);
        }
        return value.intValue();
    }

}
