/**
 * 
 * Copyright 2005 Protique Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/

package org.activemq.transport.openwire;
import EDU.oswego.cs.dl.util.concurrent.Executor;
import org.activemq.transport.openwire.io.OpenWireFormat;
import org.activemq.transport.tcp.TcpTransportChannel;
import org.activemq.transport.tcp.TcpTransportServerChannel;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.jms.JMSException;
import java.net.Socket;
import java.net.URI;

/**
 * A transport for using the <a href="http://activemq.org/OpenWire">OpenWire</a> transport protocol.
 *
 * @version $Revision: 1.1 $
 */
public class OpenWireTransportChannel extends TcpTransportChannel {
    private static final Log log = LogFactory.getLog(OpenWireTransportChannel.class);

    public OpenWireTransportChannel() {
        super(new OpenWireFormat());
    }

    public OpenWireTransportChannel(URI remoteLocation) throws JMSException {
        super(new OpenWireFormat(), remoteLocation);
    }

    public OpenWireTransportChannel(URI remoteLocation, URI localLocation) throws JMSException {
        super(new OpenWireFormat(), remoteLocation, localLocation);
    }

    public OpenWireTransportChannel(TcpTransportServerChannel serverChannel, Socket socket, Executor executor)
            throws JMSException {
        super(serverChannel, new OpenWireFormat(), socket, executor);
    }

    public OpenWireTransportChannel(Socket socket, Executor executor) throws JMSException {
        super(new OpenWireFormat(), socket, executor);
    }
}
