import org.activemq.openwire.tool.OpenWireScript

/**
 * Generates the Java test cases  for the Open Wire Format
 *
 * @version $Revision: 1.1.1.1 $
 */
class GenerateJavaTestCases extends OpenWireScript {

    Object run() {
        def destDir = new File("src/test/org/activemq/transport/openwire/io")
       destDir.mkdirs()

       def messageClasses = classes.findAll { isMessageType(it) }

       println "Generating Java marshalling code to directory ${destDir}"

       int counter = 0
       Map map = [:]

       for (jclass in messageClasses) {

           println "Processing $jclass.simpleName"

           def properties = jclass.declaredProperties.findAll { isValidProperty(it) }
           def file = new File(destDir, jclass.simpleName + "MarshallerTest.java")

           String baseClass = "PacketMarshallerTestSupport"
           if (jclass.superclass?.simpleName == "ActiveMQMessage") {
               baseClass = "ActiveMQMessageMarshallerTest"
           }

           file.withWriter { out |
               out << """/**
* Test case for the marshalling code for Open Wire Format for ${jclass.simpleName}
*
*
* NOTE!: This file is autogenerated - do not modify!
*        if you need to make a change, please see the Groovy scripts in the
*        activemq-openwire module
*/

package org.activemq.transport.openwire.io;

import org.activemq.io.impl.AbstractPacketMarshaller;
import org.activemq.message.*;
import org.activemq.message.Packet;
import org.activemq.transport.openwire.PacketMarshallerTestSupport;

/**
 * AUTOGENERATED test case for OpenWire command ${jclass.simpleName}

 * @version \$Revision: 1.1 \$
 */
public class ${jclass.simpleName}MarshallerTest extends $baseClass {

    /**
     * @return a newly created marshaller
     */
    protected AbstractPacketMarshaller createMarshaller() {
        return new ${jclass.simpleName}Marshaller();
    }


    /**
     * configures the packet with some test data
     */
    protected void populatePacket(Packet packet) throws Exception {
        super.populatePacket(packet);
        ${jclass.simpleName} info = (${jclass.simpleName}) packet;
"""
               def buffer = new StringBuffer()

               for (property in properties) {
                   def type = property.type.qualifiedName

                   def value = createTestValue(jclass, property)

                   out << """
        info.${property.setter.simpleName}($value);"""

                     buffer.append("""
        assertEquals("property $property.simpleName", $value, info.${property.getter.simpleName}());""")
               }

               out << """
   }

    /**
     * Asserts that the read packet has the correct values
     */
    protected void assertPacketValid(Packet packet) throws Exception {
        super.assertPacketValid(packet);
        ${jclass.simpleName} info = (${jclass.simpleName}) packet;
        $buffer
    }
}
"""
           }
       }
   }
}