import org.activemq.openwire.tool.OpenWireScript

/**
 * Generates the Ruby classes for the Open Wire Format
 *
 * @version $Revision: 1.1.1.1 $
 */
class GenerateRubyClasses extends OpenWireScript {

    Object run() {
        def destDir = new File("language/ruby/src/org/activemq/openwire")
        destDir.mkdirs()

        def messageClasses = classes.findAll { isMessageType(it) }

        println "Generating Java marshalling code to directory ${destDir}"

        def buffer = new StringBuffer()

        int counter = 0
        Map map = [:]

        for (jclass in messageClasses) {

            println "Processing $jclass.simpleName"

            def properties = jclass.declaredProperties.findAll { isValidProperty(it) }
            def file = new File(destDir, jclass.simpleName + ".rb")

            String baseClass = "AbstractPacket"
            if (jclass.superclass?.simpleName == "ActiveMQMessage") {
                baseClass = "ActiveMQMessage"
            }

            buffer << """
${jclass.simpleName}.class
"""

            file.withWriter { out |
                out << """#
# Marshalling code for Open Wire Format for ${jclass.simpleName}
#
#
# NOTE!: This file is autogenerated - do not modify!
#        if you need to make a change, please see the Groovy scripts in the
#        activemq-openwire module
#

class ${jclass.simpleName} < $baseClass

"""
                for (property in properties) {

                    def type = toRubyType(property.type)
                    def name = decapitalize(property.simpleName)
                    out << """
    attr_accessor : $name   # of type $type;"""
                }

                out << """


    def packetType()
        return ${getEnum(jclass)};
    end

end
"""
            }
        }
    }

    def toRubyType(type) {
        def name = type.qualifiedName
        switch (type) {
            case "java.lang.String":
                return "String"
            case "org.activemq.message.ActiveMQDestination":
                return "ActiveMQDestination"
            case "org.activemq.message.ActiveMQXid":
                return "ActiveMQXid"
            default:
                return name
        }
    }
}